==> spandsp-0.0.3.tgz: spandsp-0.0.3/tests/t38_core_tests.c <==
/*
 * SpanDSP - a series of DSP components for telephony
 *
 * t38_core_tests.c - Tests for the T.38 FoIP core module.
 *
 * Written by Steve Underwood <steveu@coppice.org>
 *
 * Copyright (C) 2007 Steve Underwood
 *
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: t38_core_tests.c,v 1.6 2007/01/23 13:53:16 steveu Exp $
 */

/*! \file */

/*! \page t38_core_tests_page T.38 core tests
\section t38_core_tests_page_sec_1 What does it do?
These tests exercise the T.38 core ASN.1 processing code.
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <inttypes.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#if defined(HAVE_TGMATH_H)
#include <tgmath.h>
#endif
#if defined(HAVE_MATH_H)
#include <math.h>
#endif
#include <assert.h>
#include <errno.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#if !defined(__USE_MISC)
#define __USE_MISC
#endif
#include <arpa/inet.h>
#include <sys/select.h>
#include <sys/time.h>
#include <audiofile.h>
#include <tiffio.h>

#include "spandsp.h"

int t38_version;
int succeeded = TRUE;
int ok_indicator_packets;
int bad_indicator_packets;
int ok_data_packets;
int bad_data_packets;
int missing_packets;

int current_indicator;
int current_data_type;
int current_field_type;
int skip;

static int rx_missing_handler(t38_core_state_t *s, void *user_data, int rx_seq_no, int expected_seq_no)
{
    missing_packets++;
    //printf("Hit missing\n");
    return 0;
}
/*- End of function --------------------------------------------------------*/

static int rx_indicator_handler(t38_core_state_t *s, void *user_data, int indicator)
{
    if (indicator == current_indicator)
        ok_indicator_packets++;
    else
        bad_indicator_packets++;
    //printf("Hit indicator %d\n", indicator);
    return 0;
}
/*- End of function --------------------------------------------------------*/

static int rx_data_handler(t38_core_state_t *s, void *user_data, int data_type, int field_type, const uint8_t *buf, int len)
{
    if (--skip >= 0)
    {
        if (data_type == current_data_type  &&  field_type == current_field_type)
            ok_data_packets++;
        else
            bad_data_packets++;
    }
    else
    {
        if (data_type == current_data_type  &&  field_type == T38_FIELD_T4_NON_ECM_SIG_END)
            ok_data_packets++;
        else
            bad_data_packets++;
    }
    //printf("Hit data %d, field %d\n", data_type, field_type);
    return 0;
}
/*- End of function --------------------------------------------------------*/

static int tx_packet_handler(t38_core_state_t *s, void *user_data, const uint8_t *buf, int len, int count)
{
    t38_core_state_t *t;
    static int seq_no = 0;
    
    t = (t38_core_state_t *) user_data;
    span_log(&s->logging, SPAN_LOG_FLOW, "Send seq %d, len %d, count %d\n", s->tx_seq_no, len, count);
    if (t38_core_rx_ifp_packet(t, seq_no, buf, len) < 0)
        succeeded = FALSE;
    seq_no++;
    return 0;
}
/*- End of function --------------------------------------------------------*/

int main(int argc, char *argv[])
{
    t38_core_state_t t38_core_a;
    t38_core_state_t t38_core_b;
    t38_data_field_t field[42];
    int i;
    int j;

    for (t38_version = 0;  t38_version < 2;  t38_version++)
    {
        printf("Using T.38 version %d\n", t38_version);

        if (t38_core_init(&t38_core_a,
                          rx_indicator_handler,
                          rx_data_handler,
                          rx_missing_handler,
                          &t38_core_b,
                          tx_packet_handler,
                          &t38_core_b) == NULL)
        {
            fprintf(stderr, "Cannot start the T.38 core\n");
            exit(2);
        }
        if (t38_core_init(&t38_core_b,
                          rx_indicator_handler,
                          rx_data_handler,
                          rx_missing_handler,
                          &t38_core_a,
                          tx_packet_handler,
                          &t38_core_a) == NULL)
        {
            fprintf(stderr, "Cannot start the T.38 core\n");
            exit(2);
        }

        t38_set_t38_version(&t38_core_a, t38_version);
        t38_set_t38_version(&t38_core_b, t38_version);

        span_log_set_level(&t38_core_a.logging, SPAN_LOG_DEBUG | SPAN_LOG_SHOW_TAG);
        span_log_set_tag(&t38_core_a.logging, "T.38-A");
        span_log_set_level(&t38_core_b.logging, SPAN_LOG_DEBUG | SPAN_LOG_SHOW_TAG);
        span_log_set_tag(&t38_core_b.logging, "T.38-B");

        ok_indicator_packets = 0;
        bad_indicator_packets = 0;
        ok_data_packets = 0;
        bad_data_packets = 0;
        missing_packets = 0;

        /* Try all the indicator types */
        for (i = 0;  i < 100;  i++)
        {
            current_indicator = i;
            if (t38_core_send_indicator(&t38_core_a, i, 3) < 0)
                break;
        }

        /* Try all the data types, as single field messages with no data */
        for (i = 0;  i < 100;  i++)
        {
            for (j = 0;  j < 100;  j++)
            {
                current_data_type = i;
                current_field_type = j;
                skip = 99;
                if (t38_core_send_data(&t38_core_a, i, j, (uint8_t *) "", 0, 1) < 0)
                    break;
            }
            if (j == 0)
                break;
        }

        /* Try all the data types and field types, as single field messages with data */
        for (i = 0;  i < 100;  i++)
        {
            for (j = 0;  j < 100;  j++)
            {
                current_data_type = i;
                current_field_type = j;
                skip = 99;
                if (t38_core_send_data(&t38_core_a, i, j, (uint8_t *) "ABCD", 4, 1) < 0)
                    break;
            }
            if (j == 0)
                break;
        }

        /* Try all the data types and field types, as multi-field messages */
        for (i = 0;  i < 100;  i++)
        {
            for (j = 0;  j < 100;  j++)
            {
                current_data_type = i;
                current_field_type = j;
                skip = 1;

                field[0].field_type = j;
                field[0].field = (const uint8_t *) "123456789ABCDEFG";
                field[0].field_len = 16;
                field[1].field_type = T38_FIELD_T4_NON_ECM_SIG_END;
                field[1].field = (const uint8_t *) "123456";
                field[1].field_len = 6;
                if (t38_core_send_data_multi_field(&t38_core_a, i, field, 2, 1) < 0)
                    break;
            }
            if (j == 0)
                break;
        }
        printf("Indicator packets: OK = %d, bad = %d\n", ok_indicator_packets, bad_indicator_packets);
        printf("Data packets: OK = %d, bad = %d\n", ok_data_packets, bad_data_packets);
        printf("Missing packets = %d\n", missing_packets);
        if (t38_version == 0)
        {
            if (ok_indicator_packets != 16  ||  bad_indicator_packets != 0)
            {
                printf("Tests failed\n");
                exit(2);
            }
            if (ok_data_packets != 288  ||  bad_data_packets != 0)
            {
                printf("Tests failed\n");
                exit(2);
            }
        }
        else
        {
            if (ok_indicator_packets != 23  ||  bad_indicator_packets != 0)
            {
                printf("Tests failed\n");
                exit(2);
            }
            if (ok_data_packets != 720  ||  bad_data_packets != 0)
            {
                printf("Tests failed\n");
                exit(2);
            }
        }
        if (missing_packets > 0)
        {
            printf("Tests failed\n");
            exit(2);
        }
    }
    if (!succeeded)
    {
        printf("Tests failed\n");
        exit(2);
    }
    printf("Tests passed\n");
    return  0;
}
/*- End of function --------------------------------------------------------*/
/*- End of file ------------------------------------------------------------*/

==> spandsp-0.0.4pre3.tgz: spandsp-0.0.4/tests/t38_core_tests.c <==
/*
 * SpanDSP - a series of DSP components for telephony
 *
 * t38_core_tests.c - Tests for the T.38 FoIP core module.
 *
 * Written by Steve Underwood <steveu@coppice.org>
 *
 * Copyright (C) 2007 Steve Underwood
 *
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: t38_core_tests.c,v 1.7 2007/04/03 12:59:33 steveu Exp $
 */

/*! \file */

/*! \page t38_core_tests_page T.38 core tests
\section t38_core_tests_page_sec_1 What does it do?
These tests exercise the T.38 core ASN.1 processing code.
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <inttypes.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#if defined(HAVE_TGMATH_H)
#include <tgmath.h>
#endif
#if defined(HAVE_MATH_H)
#include <math.h>
#endif
#include <assert.h>
#include <errno.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#if !defined(__USE_MISC)
#define __USE_MISC
#endif
#include <arpa/inet.h>
#include <sys/select.h>
#include <sys/time.h>
#include <audiofile.h>
#include <tiffio.h>

#include "spandsp.h"

int t38_version;
int succeeded = TRUE;
int ok_indicator_packets;
int bad_indicator_packets;
int ok_data_packets;
int bad_data_packets;
int missing_packets;

int current_indicator;
int current_data_type;
int current_field_type;
int skip;

static int rx_missing_handler(t38_core_state_t *s, void *user_data, int rx_seq_no, int expected_seq_no)
{
    missing_packets++;
    //printf("Hit missing\n");
    return 0;
}
/*- End of function --------------------------------------------------------*/

static int rx_indicator_handler(t38_core_state_t *s, void *user_data, int indicator)
{
    if (indicator == current_indicator)
        ok_indicator_packets++;
    else
        bad_indicator_packets++;
    //printf("Hit indicator %d\n", indicator);
    return 0;
}
/*- End of function --------------------------------------------------------*/

static int rx_data_handler(t38_core_state_t *s, void *user_data, int data_type, int field_type, const uint8_t *buf, int len)
{
    if (--skip >= 0)
    {
        if (data_type == current_data_type  &&  field_type == current_field_type)
            ok_data_packets++;
        else
            bad_data_packets++;
    }
    else
    {
        if (data_type == current_data_type  &&  field_type == T38_FIELD_T4_NON_ECM_SIG_END)
            ok_data_packets++;
        else
            bad_data_packets++;
    }
    //printf("Hit data %d, field %d\n", data_type, field_type);
    return 0;
}
/*- End of function --------------------------------------------------------*/

static int tx_packet_handler(t38_core_state_t *s, void *user_data, const uint8_t *buf, int len, int count)
{
    t38_core_state_t *t;
    static int seq_no = 0;
    
    t = (t38_core_state_t *) user_data;
    span_log(&s->logging, SPAN_LOG_FLOW, "Send seq %d, len %d, count %d\n", s->tx_seq_no, len, count);
    if (t38_core_rx_ifp_packet(t, buf, len, seq_no) < 0)
        succeeded = FALSE;
    seq_no++;
    return 0;
}
/*- End of function --------------------------------------------------------*/

int main(int argc, char *argv[])
{
    t38_core_state_t t38_core_a;
    t38_core_state_t t38_core_b;
    t38_data_field_t field[42];
    int i;
    int j;

    for (t38_version = 0;  t38_version < 2;  t38_version++)
    {
        printf("Using T.38 version %d\n", t38_version);

        if (t38_core_init(&t38_core_a,
                          rx_indicator_handler,
                          rx_data_handler,
                          rx_missing_handler,
                          &t38_core_b,
                          tx_packet_handler,
                          &t38_core_b) == NULL)
        {
            fprintf(stderr, "Cannot start the T.38 core\n");
            exit(2);
        }
        if (t38_core_init(&t38_core_b,
                          rx_indicator_handler,
                          rx_data_handler,
                          rx_missing_handler,
                          &t38_core_a,
                          tx_packet_handler,
                          &t38_core_a) == NULL)
        {
            fprintf(stderr, "Cannot start the T.38 core\n");
            exit(2);
        }

        t38_set_t38_version(&t38_core_a, t38_version);
        t38_set_t38_version(&t38_core_b, t38_version);

        span_log_set_level(&t38_core_a.logging, SPAN_LOG_DEBUG | SPAN_LOG_SHOW_TAG);
        span_log_set_tag(&t38_core_a.logging, "T.38-A");
        span_log_set_level(&t38_core_b.logging, SPAN_LOG_DEBUG | SPAN_LOG_SHOW_TAG);
        span_log_set_tag(&t38_core_b.logging, "T.38-B");

        ok_indicator_packets = 0;
        bad_indicator_packets = 0;
        ok_data_packets = 0;
        bad_data_packets = 0;
        missing_packets = 0;

        /* Try all the indicator types */
        for (i = 0;  i < 100;  i++)
        {
            current_indicator = i;
            if (t38_core_send_indicator(&t38_core_a, i, 3) < 0)
                break;
        }

        /* Try all the data types, as single field messages with no data */
        for (i = 0;  i < 100;  i++)
        {
            for (j = 0;  j < 100;  j++)
            {
                current_data_type = i;
                current_field_type = j;
                skip = 99;
                if (t38_core_send_data(&t38_core_a, i, j, (uint8_t *) "", 0, 1) < 0)
                    break;
            }
            if (j == 0)
                break;
        }

        /* Try all the data types and field types, as single field messages with data */
        for (i = 0;  i < 100;  i++)
        {
            for (j = 0;  j < 100;  j++)
            {
                current_data_type = i;
                current_field_type = j;
                skip = 99;
                if (t38_core_send_data(&t38_core_a, i, j, (uint8_t *) "ABCD", 4, 1) < 0)
                    break;
            }
            if (j == 0)
                break;
        }

        /* Try all the data types and field types, as multi-field messages */
        for (i = 0;  i < 100;  i++)
        {
            for (j = 0;  j < 100;  j++)
            {
                current_data_type = i;
                current_field_type = j;
                skip = 1;

                field[0].field_type = j;
                field[0].field = (const uint8_t *) "123456789ABCDEFG";
                field[0].field_len = 16;
                field[1].field_type = T38_FIELD_T4_NON_ECM_SIG_END;
                field[1].field = (const uint8_t *) "123456";
                field[1].field_len = 6;
                if (t38_core_send_data_multi_field(&t38_core_a, i, field, 2, 1) < 0)
                    break;
            }
            if (j == 0)
                break;
        }
        printf("Indicator packets: OK = %d, bad = %d\n", ok_indicator_packets, bad_indicator_packets);
        printf("Data packets: OK = %d, bad = %d\n", ok_data_packets, bad_data_packets);
        printf("Missing packets = %d\n", missing_packets);
        if (t38_version == 0)
        {
            if (ok_indicator_packets != 16  ||  bad_indicator_packets != 0)
            {
                printf("Tests failed\n");
                exit(2);
            }
            if (ok_data_packets != 288  ||  bad_data_packets != 0)
            {
                printf("Tests failed\n");
                exit(2);
            }
        }
        else
        {
            if (ok_indicator_packets != 23  ||  bad_indicator_packets != 0)
            {
                printf("Tests failed\n");
                exit(2);
            }
            if (ok_data_packets != 720  ||  bad_data_packets != 0)
            {
                printf("Tests failed\n");
                exit(2);
            }
        }
        if (missing_packets > 0)
        {
            printf("Tests failed\n");
            exit(2);
        }
    }
    if (!succeeded)
    {
        printf("Tests failed\n");
        exit(2);
    }
    printf("Tests passed\n");
    return  0;
}
/*- End of function --------------------------------------------------------*/
/*- End of file ------------------------------------------------------------*/
